@@ -9,15 +9,14 @@
  *   'app'  --  The application which is providing authentication.
  *
  *
- * $Horde: framework/Auth/Auth/application.php,v 1.27 2004/07/04 19:47:49 jan Exp $
+ * $Horde: framework/Auth/Auth/application.php,v 1.27.10.4 2005/01/03 12:18:52 jan Exp $
  *
- * Copyright 2002-2004 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2002-2005 Chuck Hagenbuch <chuck@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Chuck Hagenbuch <chuck@horde.org>
- * @version $Revision: 1.27 $
  * @since   Horde 3.0
  * @package Horde_Auth
  */
@@ -48,6 +47,33 @@ class Auth_application extends Auth {
         $this->_setParams($params);
     }
 
+    /**
+     * Queries the current Auth object to find out if it supports the given
+     * capability.
+     *
+     * @access public
+     *
+     * @param string $capability  The capability to test for.
+     *
+     * @return boolean  Whether or not the capability is supported.
+     */
+    function hasCapability($capability)
+    {
+        static $loaded = array();
+
+        $methods = array('list' => 'userList',
+                         'add' => 'addUser',
+                         'update' => 'updateUser',
+                         'remove' => 'removeUser');
+
+        if (empty($loaded[$capability]) && isset($methods[$capability])) {
+            $this->capabilities[$capability] = $GLOBALS['registry']->hasMethod($methods[$capability], $this->_params['app']);
+            $loaded[$capability] = true;
+        }
+
+        return !empty($this->capabilities[$capability]);
+    }
+
     /**
      * Set connection parameters.
      *
@@ -57,19 +83,13 @@ class Auth_application extends Auth {
      */
     function _setParams($params)
     {
-        global $registry;
-
         Horde::assertDriverConfig($params, 'auth',
                                   array('app'),
                                   'authentication application');
 
-        if (empty($registry->applications[$params['app']])) {
+        if (empty($GLOBALS['registry']->applications[$params['app']])) {
             Horde::fatal(PEAR::raiseError($params['app'] . ' is not configured in the Horde Registry.'), __FILE__, __LINE__);
         }
-        $this->capabilities['list']   = $registry->hasMethod('userList', $params['app']);
-        $this->capabilities['add']    = $registry->hasMethod('addUser', $params['app']);
-        $this->capabilities['update'] = $registry->hasMethod('updateUser', $params['app']);
-        $this->capabilities['remove'] = $registry->hasMethod('removeUser', $params['app']);
 
         $this->_params = $params;
     }
@@ -86,15 +106,13 @@ class Auth_application extends Auth {
      */
     function _authenticate($userId, $credentials)
     {
-        global $registry;
-
-        if (!$registry->hasMethod('authenticate', $this->_params['app'])) {
+        if (!$GLOBALS['registry']->hasMethod('authenticate', $this->_params['app'])) {
             Horde::fatal(PEAR::raiseError($this->_params['app'] . ' does not provide an authenticate() method.'), __FILE__, __LINE__);
         }
-        return $registry->callByPackage($this->_params['app'], 'authenticate',
-                                        array('userId' => $userId,
-                                              'credentials' => $credentials,
-                                              'params' => $this->_params));
+        return $GLOBALS['registry']->callByPackage($this->_params['app'], 'authenticate',
+                                                   array('userId' => $userId,
+                                                         'credentials' => $credentials,
+                                                         'params' => $this->_params));
     }
 
     /**
