@@ -50,12 +50,12 @@ if (!defined('LATT_HASCHILDREN')) {
  * functions to iterate through this tree and query information about
  * individual mailboxes/folders.
  *
- * $Horde: framework/IMAP/IMAP/Tree.php,v 1.48 2004/12/18 20:07:48 slusarz Exp $
+ * $Horde: framework/IMAP/IMAP/Tree.php,v 1.48.2.2 2005/01/03 12:19:02 jan Exp $
  *
- * Copyright 2000-2004 Chuck Hagenbuch <chuck@horde.org>
- * Copyright 2000-2004 Jon Parise <jon@horde.org>
- * Copyright 2000-2004 Anil Madhavapeddy <avsm@horde.org>
- * Copyright 2003-2004 Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * Copyright 2000-2005 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2000-2005 Jon Parise <jon@horde.org>
+ * Copyright 2000-2005 Anil Madhavapeddy <avsm@horde.org>
+ * Copyright 2003-2005 Michael Slusarz <slusarz@bigworm.colorado.edu>
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
@@ -64,7 +64,7 @@ if (!defined('LATT_HASCHILDREN')) {
  * @author  Jon Parise <jon@horde.org>
  * @author  Anil Madhavapeddy <avsm@horde.org>
  * @author  Michael Slusarz <slusarz@bigworm.colorado.edu>
- * @version $Revision: 1.48 $
+ * @version $Revision: 1.48.2.2 $
  * @since   Horde 3.0
  * @package Horde_IMAP
  */
@@ -1053,12 +1053,10 @@ class IMAP_Tree {
      *
      * @access private
      *
-     * @param array $list               A list of stdClass objects in the
-     *                                  format retuned from
-     *                                  imap_getmailboxes().
-     * @param optional boolean $expand  Expand subfolders?
+     * @param array $list  A list of stdClass objects in the format returned
+     *                     from imap_getmailboxes().
      */
-    function _addLevel($list, $expand = true)
+    function _addLevel($list)
     {
         $expandall = ($this->_initmode & IMAPTREE_INIT_FETCHALL);
         foreach ($list as $key => $val) {
@@ -1078,12 +1076,10 @@ class IMAP_Tree {
                 }
             }
 
-            if ($expand || $expandall) {
-                if ($expandall || $this->isOpen($elt)) {
-                    $this->expand($elt['v']);
-                } else {
-                    $this->_childrenInfo($elt['v'], false, true);
-                }
+            if ($expandall || $this->isOpen($elt)) {
+                $this->expand($elt['v']);
+            } else {
+                $this->_childrenInfo($elt['v'], false, true);
             }
         }
 
@@ -1591,7 +1587,7 @@ class IMAP_Tree {
         if (!$tried || ($info['haschildren'] && $list)) {
             if (($this->_mode == IMAPTREE_MODE_MAIL) &&
                 (strcasecmp($id, 'INBOX') == 0) &&
-                (!empty($this->_prefix) && !empty($this->_namespace))) {
+                (!empty($this->_prefix) || !empty($this->_namespace))) {
                 if (!empty($this->_prefix)) {
                     $query = $this->_prefix . '%';
                 } else {
