@@ -47,15 +47,15 @@ define('MIME_STRING', 2);
 /**
  * The MIME:: class provides methods for dealing with MIME standards.
  *
- * $Horde: framework/MIME/MIME.php,v 1.139 2004/12/01 02:21:10 slusarz Exp $
+ * $Horde: framework/MIME/MIME.php,v 1.139.4.2 2005/01/04 03:01:40 slusarz Exp $
  *
- * Copyright 1999-2004 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 1999-2005 Chuck Hagenbuch <chuck@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Chuck Hagenbuch <chuck@horde.org>
- * @version $Revision: 1.139 $
+ * @version $Revision: 1.139.4.2 $
  * @since   Horde 1.3
  * @package Horde_MIME
  */
@@ -660,10 +660,13 @@ class MIME {
         $from = $parser->parseAddressList($address, $defserver, false, false);
 
         foreach ($from as $entry) {
-            if (isset($entry->mailbox) && isset($entry->host)) {
-                $addressList[] = $entry->mailbox . '@' . $entry->host;
-            } elseif (isset($entry->mailbox)) {
-                $addressList[] = $entry->mailbox;
+            if (isset($entry->mailbox) &&
+                ($entry->mailbox != 'undisclosed-recipients')) {
+                if (isset($entry->host)) {
+                    $addressList[] = $entry->mailbox . '@' . $entry->host;
+                } else {
+                    $addressList[] = $entry->mailbox;
+                }
             }
         }
 
