@@ -8,10 +8,10 @@
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
- * $Horde: framework/Tree/Tree/javascript.php,v 1.34.2.1 2005/01/03 12:19:18 jan Exp $
+ * $Horde: framework/Tree/Tree/javascript.php,v 1.34.2.2 2005/01/05 10:33:01 jan Exp $
  *
  * @author  Marko Djukic <marko@oblo.com>
- * @version $Revision: 1.34.2.1 $
+ * @version $Revision: 1.34.2.2 $
  * @package Horde_Tree
  * @since   Horde 3.0
  */
@@ -55,7 +55,8 @@ class Horde_Tree_javascript extends Horde_Tree {
         /* Check for a javascript session state. */
         if ($this->_usesession &&
             isset($_COOKIE[$this->_instance . '_expanded'])) {
-            $nodes = explode(',', $_COOKIE[$this->_instance . '_expanded']);
+            /* Remove "exp" prefix from cookie value. */
+            $nodes = explode(',', substr($_COOKIE[$this->_instance . '_expanded'], 3));
 
             /* Make sure there are no previous nodes stored in the
              * session. */
